@@ -292,6 +292,9 @@ data ElaboratedConfiguredPackage
        elabHaddockQuickJump      :: Bool,
        elabHaddockHscolourCss    :: Maybe FilePath,
        elabHaddockContents       :: Maybe PathTemplate,
+       elabHaddockIndex          :: Maybe PathTemplate,
+       elabHaddockBaseUrl        :: Maybe String,
+       elabHaddockLib            :: Maybe String,
 
        elabTestMachineLog        :: Maybe PathTemplate,
        elabTestHumanLog          :: Maybe PathTemplate,
